/*
 * Copyright
 *   2009 axYus - www.axyus.com
 *   2009 C.Bosquet - charles.bosquet@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.dgfip.xemelios.batch;

import fr.gouv.finances.dgfip.xemelios.data.impl.pool.PoolManager;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

/**
 * Classe regroupant les elements communs aux traitements batch
 * @author CBO
 */
public abstract class Batch extends AbstractBatch {
    private Properties props;

    private static final String EXECUTION_FORCEE = "-force";
//    private static final String TRACE_DEBUT_TRAITEMENT = "Dmarrage du traitement le {0}  {1}";
//    private static final String TRACE_FIN_TRAITEMENT = "Fin du traitement le {0}  {1}";
//    private static final String TRACE_INTERRUPTION_ERREUR_INATTENDUE = "Une erreur inattendue  interrompu le traitement";
//  private TraitementBO traitement;

    /**
     * @param args Arguments ventuels de la ligne de commande
     */
    public Batch(String args[]) {
        super(args);
    }

    @Override
    protected void initialize() throws Exception {
        super.initialize();
        checkDirectories();
    }

    protected void checkDirectories() throws Exception {
//      DirectoryUtils.checkDirectories();
    }

    @Override
    protected void processTerminaison() throws Exception {
        Connection connection = null;
        try {
            connection = PoolManager.getInstance().getConnection();
//      JdbcUtils.executeUpdate(connection, "call pck_demat_tache_utils.termineTache(?)", new SimplePStmtBinderBuilder().add(typeTraitementRefCode()).toPStmtBinder());
        } finally {
            PoolManager.getInstance().releaseConnection(connection);
        }
    }

    @Override
    protected boolean canStart() throws Exception {
//    Connection connection = null;
//    CallableStatement callableStatement = null;
//    String executionForcee = "0";
//    try {
//      connection = PoolManager.getInstance().getConnection();
//      callableStatement = connection.prepareCall("{ ?  = call pck_demat_tache_utils.lanceTache(?, ?)}");
//      callableStatement.registerOutParameter(1, Types.INTEGER);
//      callableStatement.setString(2, typeTraitementRefCode());
//      if(isForced()) {
//        executionForcee="1";
//      }
//      callableStatement.setString(3, executionForcee);
//
//      callableStatement.execute();
//      return callableStatement.getInt(1) == 1;
//    } finally {
//      try {
//        if (callableStatement != null) {
//          callableStatement.close();
//        }
//      } finally {
//        PoolManager.getInstance().releaseConnection(connection);
//      }
//    }
        return true;
    }

    public boolean isForced() {
        if (getArgs() != null) {
            for (String arg : getArgs()) {
                if (arg.equalsIgnoreCase(EXECUTION_FORCEE)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Implmentation du traitement
     */
    protected abstract void doProcess() throws Exception;

    /**
     * @return Resum du traitement exemple: 35 fichiers traites avec succs, 3 dossiers en erreur
     */
    abstract public String getResumeTraitement();

    /**
     * @return Le refCode du type de traitement
     */
    abstract public String typeTraitementRefCode();

//  protected TraitementBO getTraitement() {
//    return this.traitement;
//  }
    @Override
    protected void process() throws Exception {
        startRapportTraitements();
        try {
            doProcess();
        } catch (Throwable e) {
//      TraitementManager.traceMessage(traitement, Gravite.ERREUR_BLOQUANTE, TRACE_INTERRUPTION_ERREUR_INATTENDUE, e);
        } finally {
            endRapportTraitements();
        }
    }

    /**
     * Dmarre le rapport du traitement
     */
    private void startRapportTraitements() throws SQLException {
//    traitement = TraitementManager.debuteTraitement(typeTraitementRefCode());
//    Date now = new Date();
//    String messageDeDemarrage = StringUtils.formatPattern(TRACE_DEBUT_TRAITEMENT, DateUtils.formatDate(now), DateUtils.formatTime(now));
//    TraitementManager.traceMessage(traitement, Gravite.INFORMATION, messageDeDemarrage);
    }

    /**
     *  Met fin au rapport traitememt
     */
    private void endRapportTraitements() throws SQLException {
//    Date now = new Date();
//    String messageDeDemarrage = StringUtils.formatPattern(TRACE_FIN_TRAITEMENT, DateUtils.formatDate(now), DateUtils.formatTime(now));
//    TraitementManager.traceMessage(traitement, Gravite.INFORMATION, messageDeDemarrage);
//    TraitementManager.termineTraitement(traitement, getResumeTraitement());
    }

    public Properties getProps() {
        return props;
    }

    public void setProps(Properties props) {
        this.props = props;
    }
    

}
