/*
 * Copyright 
 *   2005 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.plugins.paye;

import fr.gouv.finances.cp.xemelios.data.impl.mysql.MySqlDataResultSet;

import java.math.BigDecimal;


import org.apache.log4j.Logger;

import fr.gouv.finances.dgfip.xemelios.data.DataLayerManager;
import fr.gouv.finances.dgfip.xemelios.data.DataResultSet;
import fr.gouv.finances.cp.xemelios.data.impl.MySqlDataLayer;
import fr.gouv.finances.dgfip.xemelios.data.impl.sqlconfig.TDocument;
import fr.gouv.finances.dgfip.xemelios.data.impl.sqlconfig.TEtat;
import fr.gouv.finances.dgfip.xemelios.auth.XemeliosUser;
import fr.gouv.finances.dgfip.xemelios.plugins.SearchPlugin;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

/**
 * Plugin permettant de trouver les agents sortants ou entrants entre 2 periodes.
 * Ce plugin cherche les agents qui sont presents dans la periode 1 (year1 - month1)
 * et pas dans la periode 2 (year2 - month2)
 * @author chm
 */
public class Dedoublonnement extends SearchPlugin {
    private static final Logger logger = Logger.getLogger(Dedoublonnement.class);
    private String year, month, type;
    private String mtNet, op_mtNet;
    private String mtBrut, op_mtBrut;
    private String agenStatut, op_agenStatut;
    private String emploi, op_emploi;
    private String service, op_service;
    private String grade, op_grade;
    private String indice, op_indice;

    private static String TYPE_RIB = "RIB";
    private static String TYPE_MATRICULE = "MATRICULE";

    public Dedoublonnement(XemeliosUser user) {
        super(user);
    }


    public void computeParameters() throws Exception {
        year = getParameters().get("VALEUR:year");
        month = getParameters().get("VALEUR:month");
        type = getParameters().get("VALEUR:critere");
        mtNet = getParameters().get("VALEUR:mtnet");
        op_mtNet = getParameters().get("OPERATEUR:mtnet");
        mtBrut = getParameters().get("VALEUR:mtbrut");
        op_mtBrut = getParameters().get("OPERATEUR:mtbrut");
        agenStatut = getParameters().get("VALEUR:agent_statut");
        op_agenStatut = "xem:StringEquals";
        emploi = getParameters().get("VALEUR:emploi");
        op_emploi = getParameters().get("OPERATEUR:emploi");
        service = getParameters().get("VALEUR:service");
        op_service = getParameters().get("OPERATEUR:service");
        grade = getParameters().get("VALEUR:grade");
        op_grade = getParameters().get("OPERATEUR:grade");
        indice = getParameters().get("VALEUR:indice");
        op_indice = getParameters().get("OPERATEUR:indice");        

        StringBuffer error = new StringBuffer();
        boolean hasError = false;
        if(year==null || year.length()==0) {
            error.append("Le champ anne doit tre rempli\n");
            hasError = true;
        }
        try {
        	if (mtNet != null && !"".equals(mtNet)) {
        		new BigDecimal (mtNet);
        	}
        } catch (NumberFormatException nfe) {
        	error.append("Le champ montant net ne contient pas un montant valide\n");
        	hasError = true;
        }
        try {
        	if (mtBrut != null && !"".equals(mtBrut)) {
        		new BigDecimal (mtBrut);
        	}
        } catch (NumberFormatException nfe) {
        	error.append("Le champ montant brut ne contient pas un montant valide\n");
        	hasError = true;
        }
        try {
        	if (indice != null && !"".equals(indice)) {
        		new BigDecimal (indice);
        	}
        } catch (NumberFormatException nfe) {
        	error.append("Le champ montant indice ne contient pas un nombre valide\n");
        	hasError = true;
        }
        if(hasError) {
            logger.error(error.toString());
            throw new Exception(error.toString());
        }
    }
    

    @Override
    public DataResultSet getResultSet() throws Exception {
        computeParameters();
// gets the persistence
        MySqlDataLayer dl = (MySqlDataLayer)DataLayerManager.getImplementation();
        TDocument pc = dl.getPersistenceConfig(getEtatModel().getParent(),getUser()).getLayer("mysql").getDocument(getEtatModel().getParent().getId());
        TEtat sec = pc.getEtat(getEtatModel().getId());

        StringBuilder request = new StringBuilder();
        request.append("SELECT ");
        if(TYPE_MATRICULE.equals(type)) request.append("AGENT_MATRICULE");
        else request.append("RIB");
        request.append (" FROM ").append(sec.getIndexTable("T1"));
        request.append(" WHERE COLLECTIVITE='").append(getCollectivite().key).append("' AND BUDGET='").append(getBudget().key);
        request.append("' AND ANNEE=").append(year);
        if (month!=null && !"RIEN".equals(month)) {
        	request.append (" AND MOIS=").append(month);
        }        
        /* MONTANT NET */
        if (mtNet!=null && !"".equals(mtNet)) {
        	request.append (" AND MT_NET ").append (op_mtNet).append (mtNet).append (" ");
        }
        /* MONTANT BRUT */
        if (mtBrut!=null && !"".equals(mtBrut)) {
        	request.append (" AND MT_BRUT ").append (op_mtBrut).append (mtBrut).append (" ");
        }
        /* Grade */
        if (grade!=null && !"".equals(grade)) {
        	request.append (" AND AGENT_GRADE ").append (convertOpString (op_grade, grade)).append (" ");
        }
        /* Statut */
        if (agenStatut!=null && !"".equals(agenStatut)) {
        	request.append (" AND AGENT_STATUT ").append (convertOpString (op_agenStatut, agenStatut)).append (" ");
        }
        /* Emploi */
        if (emploi!=null && !"".equals(emploi)) {
        	request.append (" AND AGENT_METIER ").append (convertOpString (op_emploi, emploi)).append (" ");
        }
        /* Service */
        if (service!=null && !"".equals(service)) {
        	request.append (" AND SERVICE ").append (convertOpString (op_service, service)).append (" ");
        }
        /* Indice */
        if (indice!=null && !"".equals(indice)) {
        	request.append (" AND AGENT_INDICE ").append (op_indice).append (indice).append (" ");
        }
        if (TYPE_MATRICULE.equals(type)) {
        	request.append(" GROUP BY AGENT_MATRICULE HAVING COUNT(AGENT_MATRICULE)>1 ");
        } else {
        	request.append(" GROUP BY RIB HAVING COUNT(RIB)>1 ");
        }
        
        Connection con = null;
        PreparedStatement ps = null;
        try {
            con = dl.getConnection();
logger.debug(request.toString());
            ps = con.prepareStatement(request.toString());
            ResultSet rs = ps.executeQuery();
            ArrayList<String> data = new ArrayList<String>();
            while(rs.next()) {
                String matr = rs.getString(1);
                data.add(matr);
            }
            if(data.size()>0) {
                request = new StringBuilder();
                request.append("SELECT SQL_CALC_FOUND_ROWS DOC_ID FROM ").append(sec.getIndexTable("T1"));
                request.append(" WHERE ANNEE=").append(year).append(" AND MOIS='").append(month).append("' AND ");
                if(TYPE_MATRICULE.equals(type)) request.append("AGENT_MATRICULE IN (");
                else request.append("RIB IN (");
                for(String s:data) request.append("'").append(s).append("',");
                request.deleteCharAt(request.length()-1);
                request.append(") AND COLLECTIVITE='").append(getCollectivite().key).append("' AND BUDGET='").append(getBudget().key).append("' ");
                return new MySqlDataResultSet(getElementModel(),getCollectivite(),getBudget(),"/n:DocumentPaye/n:DonneesIndiv/n:PayeIndivMensuel",request.toString(),getListeResultat(),dl,getUser());
            }
        } catch(SQLException sqlEx) {
            sqlEx.printStackTrace();
        } finally {
            if(con!=null) dl.releaseConnection(con);
        }
        return dl.getEmptyDataResultSet();
    }

}
