/*
 * Copyright 
 *   2008 axYus - www.axyus.com
 *   2008 C ESCOBAR - christel.escobar@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.gouv.finances.cp.xemelios.controls.pesv2.dep;

import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.xemelios.controls.AbstractUnitControl;
import fr.gouv.finances.cp.xemelios.controls.Anomalie;
import fr.gouv.finances.cp.xemelios.controls.IdGenerator;
import fr.gouv.finances.cp.xemelios.controls.Node;
import fr.gouv.finances.cp.xemelios.controls.pesv2.ConstantsControlsPESv2;


/**
 * Implmente le contrle N DEP25
 * 
 * Il consiste  vrifier la rgle fonctionnelle suivante :
 * Les Codes Rgie, Numros d'Emprunts ou d'Inventaire doivent tre renseigns.
 * 
 */


public class DEP25 extends AbstractUnitControl implements ConstantsControlsPESv2{
	private final static Logger logger = Logger.getLogger(DEP25.class);
	public static final transient String CTRL_ID = "DEP25";
	private Vector<Anomalie> anos = new Vector<Anomalie>();
	private Hashtable<String,Object> hParams;

	/**
	 * La balise du document gnral
	 */
	
	private String docId = null;
	private String nodeId = null;

	/**
	 * La chaine cible
	 */
	
	private static final String FIN_PIECE = "/PES_DepenseAller/Bordereau/Piece/";
	private static final String LIGNEPCE = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/";
	private static final String ON_RETURN_ANOMALIES = FIN_PIECE;

	/**
	 * Variables contenant les valeurs ncessaires  ce contrle
	 */

	private String bord_IdBord = "";
	private String bord_ExerBord = "";
	private String bord_TypBord = "";
	private String piece_IdPce = "";
	private String piece_TypPce = "";	
	private String piece_NatPce = "";	
	private String ligne_IdLigne = "";
	private String ligne_IdActif = "";
	private String ligne_IdEmpruntOrdo = "";
	private String ligne_IdRegie = "";
	
	private String ligne_IdLigne_NodeId = "";
	
		
	/**
	 * Variables devant tre remplaces dans le message-echec
	 */
	
	private static final String MSG_BORD_NUM = "#BORD_NUM#";
	private static final String MSG_BORD_EXER = "#BORD_EXER#";
	private static final String MSG_BORD_TYPBORD = "#BORD_TYPBORD#";
	private static final String MSG_PIECE_TYPPCE = "#PIECE_TYPPCE#";
	private static final String MSG_PIECE_ID = "#PIECE_IDPCE#";
	private static final String MSG_LIGNE_IDLIGNE = "#LIGNE_IDLIGNE#";
	private static final String MSG_ERREUR = "#ERREUR#";
	
	/**
	 * Chemins incomplet vers les divers lments que l'on veux lire. Ils sont incomplets dans la mesure 
	 * o on n'a pas le dbut du chemin, vu que ce contrle est utilis pour les Dpenses uniquement
	 */
	
	private static final String CHEMIN_BLOCBORDEREAU = "/PES_DepenseAller/Bordereau/BlocBordereau/";
	private static final String CHEMIN_PIECE = "/PES_DepenseAller/Bordereau/Piece";
	private static final String CHEMIN_BLOCBORDEREAU_EXER = "/PES_DepenseAller/Bordereau/BlocBordereau/Exer/";
	private static final String CHEMIN_BLOCBORDEREAU_IDBORD = "/PES_DepenseAller/Bordereau/BlocBordereau/IdBord/";
	private static final String CHEMIN_BLOCBORDEREAU_TYPBORD = "/PES_DepenseAller/Bordereau/BlocBordereau/TypBord/";
	private static final String CHEMIN_BLOCPIECE_IDPIECE = "/PES_DepenseAller/Bordereau/Piece/BlocPiece/InfoPce/IdPce/";
	private static final String CHEMIN_BLOCPIECE_TYPPIECE = "/PES_DepenseAller/Bordereau/Piece/BlocPiece/InfoPce/TypPce/";
	private static final String CHEMIN_BLOCPIECE_NATPIECE = "/PES_DepenseAller/Bordereau/Piece/BlocPiece/InfoPce/NatPce/";
	private static final String CHEMIN_BLOCLIGNEPIECE = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/BlocLignePiece/";
	private static final String CHEMIN_LIENSIDENT_IDACTIF = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/BlocLignePiece/LiensIdent/IdActif/";
	private static final String CHEMIN_LIENSIDENT_IDEMPRUNTORDO = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/BlocLignePiece/LiensIdent/IdEmpruntOrdo/";
	private static final String CHEMIN_LIENSIDENT_IDREGIE = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/BlocLignePiece/LiensIdent/IdRegie/";
	private static final String CHEMIN_INFOLIGNEPIECE = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/BlocLignePiece/InfoLignePce/";
	private static final String CHEMIN_INFOLIGNEPIECE_IDLIGNE = "/PES_DepenseAller/Bordereau/Piece/LigneDePiece/BlocLignePiece/InfoLignePce/IdLigne/";

	
	public void startDocument() throws SAXException {
		logger.info("In "+CTRL_ID);
	}

	
	public void setDocId(String docID) {
		this.docId = docID;		
	}

	
	public void startElement(String uri, String localName, String qName,Attributes atts, String xpath) {
		
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU)) {
			bord_ExerBord = bord_IdBord = bord_TypBord = "";
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_EXER)) {
			bord_ExerBord = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_IDBORD)) {
			bord_IdBord = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_TYPBORD)) {
			bord_TypBord = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_PIECE)) {
			if (!anos.isEmpty())
				anos = new Vector<Anomalie>();			
			piece_IdPce = piece_TypPce = piece_NatPce = "";			
		}		
		if(xpath.endsWith(CHEMIN_BLOCPIECE_IDPIECE)){
			piece_IdPce = atts.getValue("V");
			anos.clear();
		}
		if(xpath.endsWith(CHEMIN_BLOCPIECE_TYPPIECE)){
			piece_TypPce = atts.getValue("V");
		}
		if(xpath.endsWith(CHEMIN_BLOCPIECE_NATPIECE)){
			piece_NatPce = atts.getValue("V");
		}
		if(xpath.endsWith(CHEMIN_BLOCLIGNEPIECE)){
			ligne_IdActif = ligne_IdEmpruntOrdo = ligne_IdRegie = "";
		}
		if(xpath.endsWith(CHEMIN_LIENSIDENT_IDACTIF)){
			ligne_IdActif = atts.getValue("V");
		}
		if(xpath.endsWith(CHEMIN_LIENSIDENT_IDEMPRUNTORDO)){
			ligne_IdEmpruntOrdo = atts.getValue("V");
		}
		if(xpath.endsWith(CHEMIN_LIENSIDENT_IDREGIE)){
			ligne_IdRegie = atts.getValue("V");
		}
		if(xpath.endsWith(CHEMIN_INFOLIGNEPIECE)){
			ligne_IdLigne = "";
			ligne_IdLigne_NodeId = "";
		}
		if(xpath.endsWith(CHEMIN_INFOLIGNEPIECE_IDLIGNE)){
			ligne_IdLigne = atts.getValue("V");
			ligne_IdLigne_NodeId = atts.getValue("ano:node-id");
		}
		
	}


	public Vector<Anomalie> endElement(String uri, String localName, String qName, String xpath) throws SAXException {
		
		if(xpath.endsWith(LIGNEPCE)){
			boolean err = false;
			
			if ( piece_NatPce.equals("03") &&  ligne_IdActif.equals("")) 
				err = true;
			if ( piece_NatPce.equals("04") && ligne_IdEmpruntOrdo.equals(""))
				err = true;
			if ( piece_NatPce.equals("05") && ligne_IdRegie.equals(""))
				err = true;
				
			if(err) {	
								
					String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
					msg = msg.replaceAll(MSG_BORD_NUM, bord_IdBord);
					msg = msg.replaceAll(MSG_BORD_EXER, bord_ExerBord);
					msg = msg.replaceAll(MSG_BORD_TYPBORD, bord_TypBord);
					msg = msg.replaceAll(MSG_PIECE_TYPPCE, piece_TypPce);
					msg = msg.replaceAll(MSG_PIECE_ID, piece_IdPce);
					msg = msg.replaceAll(MSG_LIGNE_IDLIGNE, ligne_IdLigne);
																								
					if (piece_NatPce.equals("03") &&  ligne_IdActif.equals(""))
						msg = msg.replaceAll(MSG_ERREUR, "Numro d'inventaire non renseign.");
					if (piece_NatPce.equals("04") && ligne_IdEmpruntOrdo.equals(""))
						msg = msg.replaceAll(MSG_ERREUR, "Numro d'emprunt non renseign.");
					if (piece_NatPce.equals("05") && ligne_IdRegie.equals(""))
						msg = msg.replaceAll(MSG_ERREUR, "Numro de rgie non renseign.");
									
					String path="@added:primary-key='"+bord_ExerBord+"-"+bord_TypBord+"-"+bord_IdBord+"'";
					String anoId = IdGenerator.nextId();
					String libelleLien = "Bordereau "+bord_IdBord+" Mandat "+piece_IdPce;
					Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
					xslParams.put("elementId","");
					xslParams.put("mandatId",piece_IdPce);
					xslParams.put("browser-destination", "internal");
					xslParams.put("anoId", anoId);
					Anomalie ano = new Anomalie(anoId,
							getDocumentModel().getControlById(CTRL_ID).getId(),
							getDocumentModel().getControlById(CTRL_ID).getLibelle(),
							getDocumentModel().getDocumentId(),
							xpath.split("/")[1],
							BORDEREAU_DEPENSE, // lment importable
							bord_IdBord, // id lment importable
							localName,
							libelleLien,
							msg,
							getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
							getDocumentModel().getControlById(CTRL_ID).getSeverity(),
							path, // xpath vers le bordereau sous lequel se trouve l'anomalie
							xslParams); // paramtres supplmentaires pour accder  l'lment en anomalie
					
					
					ano.addNode(new Node (ligne_IdLigne_NodeId));
					anos.add(ano);
				
			}
									
			return null;
		}
		else if (xpath.endsWith(ON_RETURN_ANOMALIES)){
						
			return anos;
			
		} else {
			return null;
		}
	}

	public void endDocument() throws SAXException {}

	public void characters(char[] ch, int start, int length, String xpath) {}

	public void ignorableWhitespace(char[] ch, int start, int length,String xpath) {}

	public void startPrefixMapping(String prefix, String uri, String xpath) {}

	public void endPrefixMapping(String prefix, String xpath) {}

	public void processingInstruction(String target, String data, String xpath) {}

	public void skippedEntity(String name, String xpath) {}

	public void setParameters(Hashtable<String,Object> params) {
		this.hParams=params;
	}

	public void setDocumentLocator(Locator locator) {}
	
}
