/*
 * Copyright
 *   2009 axYus - www.axyus.com
 *   2009 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.dgfip.xemelios.web.imports;

import fr.gouv.finances.dgfip.utils.Pair;
import fr.gouv.finances.dgfip.xemelios.auth.XemeliosUser;
import fr.gouv.finances.dgfip.xemelios.common.config.DocumentModel;
import fr.gouv.finances.dgfip.xemelios.common.config.DocumentsModel;
import fr.gouv.finances.dgfip.xemelios.data.DataLayerManager;
import fr.gouv.finances.dgfip.xemelios.services.XemeliosHelper;
import fr.gouv.finances.dgfip.xemelios.web.AttributeHelper;
import fr.gouv.finances.dgfip.xemelios.web.utils.StdDispatchAction;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 *
 * @author chm
 */
public class ImportDetailAction extends StdDispatchAction {
    @Override
    protected ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
        XemeliosUser user = AttributeHelper.getUser(request.getSession());
        String importId = request.getParameter("importId");
        DocumentsModel dms = XemeliosHelper.getDocumentsModel();
        List<Pair> fileTypes = new ArrayList<Pair>();
        fileTypes.add(new Pair("archive", "Archive Xemelios"));
        fileTypes.add(new Pair("zip", "Archive zip"));
        for(DocumentModel dm:dms.getDocuments()) {
            if(!user.hasDocument(dm.getId())) continue;
            if(dm.getDisplayInMenuIf()!=null && !"true".equals(System.getProperty(dm.getDisplayInMenuIf()))) continue;
            fileTypes.add(new Pair(dm.getId(),dm.getTitre()));
        }
        request.setAttribute("fileTypes",fileTypes);
        Import ip = ImportHelper.getImport(user, importId);
        if(ip==null) {
            StringBuffer sb = new StringBuffer();
            sb.append("<p style=\"font-size: 14px; font-weight: bold; vertical-align: baseline\"><img src=\"").append(request.getContextPath()).append("/img/process-stop.png\" style=\"border-style: none;\"/>").append("<span style=\"margin-left: 12px;\">Import non trouv&eacute;.</span></p>");
            request.setAttribute("message",sb.toString());
            return mapping.findForward("error");
        } else {
            DocumentModel dm = dms.getDocumentById(ip.getFileType());
            request.setAttribute("import",ip);
            List<Pair> collectivites = null;
            if(dm!=null) {
                // TODO: mettre la hierarchie des collectivites
                collectivites = DataLayerManager.getImplementation().getCollectivites(dm, null, user);
            }
            request.setAttribute("collectivites",collectivites);
            if(ip.getCollectiviteId()!=null && ip.getCollectiviteId().length()>0) {
                List<Pair> existingBudgets = DataLayerManager.getImplementation().getBudgets(dm, new Pair(ip.getCollectiviteId(),ip.getCollectiviteLib()), user);
                TreeSet<Pair> budgets = new TreeSet<Pair>();
                for (Pair p:dm.getDefaultBudgets()) budgets.add(p);
                for(Pair p:existingBudgets) budgets.add(p);
                request.setAttribute("budgets",budgets);
            }
            return mapping.findForward("ok");
        }
    }
}
